// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;
float4x4 tWVP: WORLDVIEWPROJECTION;

//texture
texture Src <string uiname="Source";>;
sampler SrcSamp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Src);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};
//source transformation
float4x4 tTex: TEXTUREMATRIX <string uiname="Source Transform";>;


// background texture
texture Mask<string uiname="Background";>;
sampler MaskSamp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Mask);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};
 //mask transformation
float4x4 tMask: TEXTUREMATRIX <string uiname="Background Transform";>;


// movement bang texture
texture Move<string uiname="Movement Bang";>;
sampler MoveSamp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Move);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};
//mask transformation
float4x4 tMove: TEXTUREMATRIX <string uiname="Movement Transform";>;



//pin for the Reset
bool Reset <
    string uiname="Reset";
    bool uimin=0;
    bool uimax=1;
> = 0;


//pin for the Threshold
float Threshold <
    string uiname="Threshold";
    float uimin=0.0;
    float uimax=1.0;
> = 0.0f;

//pin for the Factor
float Factor <
    string uiname="Size Threshold";
    float uimin=0.0;
    float uimax=1.0;
> = 0.0f;


//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos  : POSITION;
    float2 TexCd : TEXCOORD0;
    float2 MaskCd : TEXCOORD1;
    float2 MoveCd : TEXCOORD2;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------
vs2ps VS(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0,
    float4 MaskCd : TEXCOORD1,
    float4 MoveCd : TEXCOORD2
)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
    Out.Pos = mul(PosO, tWVP);
    
    //transform texturecoordinates
    Out.TexCd = mul(TexCd, tTex);

    //transform the background texture cordinates
    Out.MaskCd = mul(TexCd, tMask);

    //transform the movement texture cordinates
    Out.MoveCd = mul(TexCd, tMove);
    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------
float4 ps_Luminance(vs2ps In): COLOR
{
    const float2 pixelSize = 0.001953125f/2;


    //source texture lookup
    float4 col = tex2D(SrcSamp, In.TexCd);
    // the mask texture lookup
    float4 maskCol = tex2D(MaskSamp, In.MaskCd + pixelSize);
    // the queue texture lookup
    float4 outColor = {0.0f,0.0f,0.0f,1.0f};

    //get luminance
    float LumCol = (0.222f * col.r + 0.707f * col.g + 0.071f * col.b);
    
    outColor.rgb = LumCol;
   return outColor;
}

float4 ps_AdaptBackground(vs2ps In): COLOR
{
    const float2 pixelSize = 0.001953125f/2;


    //source texture lookup
    float4 col = tex2D(SrcSamp, In.TexCd);
    // the background texture lookup
    float4 maskCol = tex2D(MaskSamp, In.MaskCd + pixelSize);
    //source texture lookup
    float4 moveCol = tex2D(MoveSamp, In.TexCd);
    float4 outColor = {0.0f,0.0f,0.0f,1.0f};

    //get luminance
    //float LumCol = (0.222f * col.r + 0.707f * col.g + 0.071f * col.b);
    //channel green is current bw image of frame difference shader
    float LumCol = col.g;
    
    //if no movement occurs
    if (moveCol.r < 0.05f) {
      float minMask = min (maskCol.r,LumCol);
      float maxMask = max (maskCol.g,LumCol);
      float distMask = abs(maxMask - minMask);
      outColor.r = minMask;
      outColor.g = maxMask;
      outColor.b = max (maskCol.b,distMask);
    }
    else {
      outColor = maskCol;
    }
    //outColor.a = moveBang;
    //reset buffer image
    if (Reset) {
      outColor = float4(1.0f,0.0f,0.0f,1.0f);
    }
    return outColor;
}

float4 ps_AdaptiveBgSubstraction(vs2ps In): COLOR
{
    const float2 pixelSize = 0.001953125f/2;

    //source texture lookup
    float4 col = tex2D(SrcSamp, In.TexCd);
    // the mask texture lookup
    float4 maskCol = tex2D(MaskSamp, In.MaskCd - pixelSize);
    float4 outColor = {0.0f,0.0f,0.0f,1.0f};

    //get luminance
    float LumCol = (0.222f * col.r + 0.707f * col.g + 0.071f * col.b);
    float minMask = maskCol.r;
    float maxMask = maskCol.g;
    float distMask = maskCol.b;
    //check for Foreground
    minMask = abs(minMask - LumCol);  //differenz check
    maxMask = abs(maxMask - LumCol);
    if (distMask<0.05f) distMask=+0.05f;
    if (minMask > distMask+Threshold || maxMask > distMask+Threshold) {
       //LumCol = max(minMask,maxMask);
       //outColor.rgb = LumCol;
       outColor.rgb = 1.0f;
    }
    return outColor;
}

//Erode - Outputs the minimum of the center and neighbours
float4 PS_sizeThreshold(vs2ps In): COLOR {

  float4 tmpSample = 0.0f;
  float4 minSamp = tex2D(SrcSamp, In.TexCd);
  Factor = Factor * 10;

  tmpSample = tex2D(SrcSamp, In.TexCd + 0.001953125f * float2(-1*Factor,-1*Factor));
  minSamp = min(minSamp, tmpSample);
  tmpSample = tex2D(SrcSamp, In.TexCd + 0.001953125f * float2(0,-1*Factor));
  minSamp = min(minSamp, tmpSample);
  tmpSample = tex2D(SrcSamp, In.TexCd + 0.001953125f * float2(1*Factor,-1*Factor));
  minSamp = min(minSamp, tmpSample);

  tmpSample = tex2D(SrcSamp, In.TexCd + 0.001953125f * float2(-1*Factor,0));
  minSamp = min(minSamp, tmpSample);
  tmpSample = tex2D(SrcSamp, In.TexCd + 0.001953125f * float2(1*Factor,0));
  minSamp = min(minSamp, tmpSample);

  tmpSample = tex2D(SrcSamp, In.TexCd + 0.001953125f * float2(-1*Factor,1*Factor));
  minSamp = min(minSamp, tmpSample);
  tmpSample = tex2D(SrcSamp, In.TexCd + 0.001953125f * float2(0,1*Factor));
  minSamp = min(minSamp, tmpSample);
  tmpSample = tex2D(SrcSamp, In.TexCd + 0.001953125f * float2(1*Factor,1*Factor));
  minSamp = min(minSamp, tmpSample);

  minSamp.rgb=minSamp.b;
  return minSamp;
}


//Dilate - Outputs the maximum of the center and neighbours
float4 PS_sizeRestore(vs2ps In): COLOR {

  float4 tmpSample = 0.0f;
  float4 maxSamp = tex2D(SrcSamp, In.TexCd);
  Factor = Factor * 10;

  tmpSample = tex2D(SrcSamp, In.TexCd + 0.001953125f * float2(-1*Factor,-1*Factor));
  maxSamp = max(maxSamp, tmpSample);
  tmpSample = tex2D(SrcSamp, In.TexCd + 0.001953125f * float2(0,-1*Factor));
  maxSamp = max(maxSamp, tmpSample);
  tmpSample = tex2D(SrcSamp, In.TexCd + 0.001953125f * float2(1*Factor,-1*Factor));
  maxSamp = max(maxSamp, tmpSample);

  tmpSample = tex2D(SrcSamp, In.TexCd + 0.001953125f * float2(-1*Factor,0));
  maxSamp = max(maxSamp, tmpSample);
  tmpSample = tex2D(SrcSamp, In.TexCd + 0.001953125f * float2(1*Factor,0));
  maxSamp = max(maxSamp, tmpSample);

  tmpSample = tex2D(SrcSamp, In.TexCd + 0.001953125f * float2(-1*Factor,1*Factor));
  maxSamp = max(maxSamp, tmpSample);
  tmpSample = tex2D(SrcSamp, In.TexCd + 0.001953125f * float2(0,1*Factor));
  maxSamp = max(maxSamp, tmpSample);
  tmpSample = tex2D(SrcSamp, In.TexCd + 0.001953125f * float2(1*Factor,1*Factor));
  maxSamp = max(maxSamp, tmpSample);

  maxSamp.rgb=maxSamp.b;
  return maxSamp;
}



 float4 ps_ShowMin(vs2ps In): COLOR
{
    const float2 pixelSize = 0.001953125f/2;
    // the mask texture lookup
    float4 maskCol = tex2D(MaskSamp, In.MaskCd - pixelSize);
    float4 outColor = {0.0f,0.0f,0.0f,1.0f};

    outColor.r = maskCol.r;
    return outColor;
}
 float4 ps_ShowMax(vs2ps In): COLOR
{
    const float2 pixelSize = 0.001953125f/2;
    // the mask texture lookup
    float4 maskCol = tex2D(MaskSamp, In.MaskCd - pixelSize);
    float4 outColor = {0.0f,0.0f,0.0f,1.0f};

    outColor.g = maskCol.g;
    return outColor;
}
 float4 ps_ShowDist(vs2ps In): COLOR
{
    const float2 pixelSize = 0.001953125f/2;
    // the mask texture lookup
    float4 maskCol = tex2D(MaskSamp, In.MaskCd - pixelSize);
    float4 outColor = {0.0f,0.0f,0.0f,1.0f};

    //multiply for better visibility
    outColor.rgb = saturate(maskCol.b*10);
    return outColor;
}




// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique ps_Luminance
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 ps_Luminance();
    }
}

technique AdaptBackground
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 ps_AdaptBackground();
    }
}


technique ps_ShowMin
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 ps_ShowMin();
    }
}
technique ps_ShowMax
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 ps_ShowMax();
    }
}
technique ps_ShowDist
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 ps_ShowDist();
    }
}



technique AdaptiveBgSubtraction
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 ps_AdaptiveBgSubstraction();
    }
}

technique SizeThreshold
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS_sizeThreshold();
    }
}

technique SizeRestore
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS_sizeRestore();
    }
}

technique TFixedFunction
{
    pass P0
    {
        //transforms
        WorldTransform[0]   = (tW);
        ViewTransform       = (tV);
        ProjectionTransform = (tP);

        //texturing
        Sampler[0] = (MaskSamp);
        TextureTransform[0] = (tTex);
        TexCoordIndex[0] = 0;
        TextureTransformFlags[0] = COUNT2;
        //Wrap0 = U;  // useful when mesh is round like a sphere
        
        Lighting       = FALSE;

        //shaders
        VertexShader = NULL;
        PixelShader  = NULL;
    }
}
